/*
 * @Description: kylin devctl module
 * @version: 2.0
 * @Author: wangknglong
 * @Date: 2023-05-23 10:44:17
 * @LastEditors: wangkanglong
 * @LastEditTime: 2023-06-01 14:25:47
 */
/*
 *  Kylin devctl module
 *
 *  This file contains the definitions needed for Kylin security devctl
 *  functions
 *
 *  Authors:  WKL <wangkanglong@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef _KYLIN_DEVCTL_MODULE_H_
#define _KYLIN_DEVCTL_MODULE_H_

#define KYLIN_DEVCTL_CHAR_MID		128
#define KYLIN_DEVCTL_CHAR_MAX		256
#define KYLIN_DEVCTL_USBNAME_MAX	30

/**  操作枚举 */
enum kylin_devctl_op_type {
	DEV_POLICY_ADD = 1,						///< 添加设备管控的策略
	DEV_POLICY_DEL,							///< 删除设备管控的策略
	BLACK_LIST_ADD,							///< 添加黑名单
	BLACK_LIST_DEL,							///< 删除黑名单
	USB_LIST_ADD,							///< 添加usb细粒度管控的策略
	USB_LIST_DEL,							///< 删除usb细粒度管控的策略
	USB_BIOS_SYNC							///< 同步bios策略
};

/**  黑名单类型枚举 */
typedef enum kylin_blacklist_id_type {
	KYLIN_WIRELESS_SSID = 1,				///< 热点黑名单类型
	KYLIN_NET_CARD,							///< 网卡黑名单类型
	KYLIN_BLACKLIST_TYPE_MAX
}blist_id_type;

/**  设备管控状态枚举 */
enum kylin_devctl_status {
	DEVCTL_OFF = 0,							///< 关闭状态
	DEVCTL_WARNING,							///< 开启状态，并弹窗
	DEVCTL_ENFORCING,						///< 开启状态，不弹框
	DEVCTL_STATUS_MAX
};

/**  设备权限枚举 */
enum kylin_device_perm {
	DEVICE_ENABLE = 1,						///< 启用
	DEVICE_DISABLE,							///< 禁用
	DEVICE_RO = 5,							///< 只读权限
	DEVICE_PERM_MAX
};

/**  usb优先级类型枚举 */
enum kylin_devctl_usb_priority {
	KYLIN_USB_INTF = 0,						///< usb接口
	KYLIN_USB_DEV,							///< usb设备
	KYLIN_USB_LIST,							///< 指定usb设备
	KYLIN_PRIORITY_MAX
};

/**  设备类型枚举 */
enum kylin_devctl_device_type {
	KYLIN_USB = 1,							///< usb接口
	KYLIN_SERIAL,							///< 串口
	KYLIN_PARALLEL,							///< 并口
	KYLIN_TYPEC,							///< type-c接口
	KYLIN_CDROM,							///< 光驱
	KYLIN_PRINTER,							///< 打印机
	KYLIN_WIRELESS,							///< 无线网卡
	KYLIN_ETHERNET,							///< 有线网卡
	KYLIN_BLUETOOTH,						///< 蓝牙
	KYLIN_CAMERA,							///< 摄像头
	KYLIN_SCANNER,							///< 扫描仪
	KYLIN_AUDIO,							///< 音频(目前仅管控到3.5mm jack)
	KYLIN_IEEE1394,							///< IEEE1394接口
	KYLIN_PCMCIA,							///< PCMCIA接口
	KYLIN_PCIE,								///< PCIE接口
	KYLIN_SATA,								///< SATA接口
	KYLIN_PS2,								///< ps2接口(暂不提供)
	KYLIN_VGA,								///< vga接口(暂不提供)
	KYLIN_HDMI,								///< HDMI接口
	KYLIN_DP,								///< DP接口
	KYLIN_INFRARED,							///< 红外设备
	KYLIN_MODEM,							///< 网络modem
	KYLIN_MMC,								///< mmc设备
	KYLIN_SCSI_RAID,						///< scsi_raid设备
	KYLIN_FLOPPY_DRIVE,						///< 软驱设备
	KYLIN_VIDEO,							///< 图像设备
	KYLIN_DIGITAL_CAMERA,					///< 数码相机设备
	KYLIN_MULTI_SERIAL_CARD,				///< 多串口卡
	KYLIN_WINCE,							///< wince设备(手机、ipad等)
	KYLIN_DEV_MAX
};

/**  usb设备类型枚举 */
enum kylin_devctl_usb_type {
	KYLIN_HARD_DRIVE = 1,					///< u盘(弃用)
	KYLIN_USB_CDROM,						///< usb光驱(弃用)
	KYLIN_USB_PRINTER,						///< 0x07 usb打印机
	KYLIN_USB_VIDEO,						///< 0x0e usb video设备
	KYLIN_USB_WIRELESS_CONTROLLER,			///< 0xe0 无线控制器
	KYLIN_USB_MASS_STORAGE,					///< 0x08 大容量存储
	KYLIN_USB_PER_INTERFACE,				///< 0x00 保留分类
	KYLIN_USB_AUDIO,						///< 0x01 音频设备
	KYLIN_USB_COMM,							///< 0x02 通信及通信类控制设备
	KYLIN_USB_HID,							///< 0x03 人体工学设备
	KYLIN_USB_PHYSICAL,						///< 0x05 物理设备
	KYLIN_USB_STILL_IMAGE,					///< 0x06 图像
	KYLIN_USB_HUB,							///< 0x09 hub设备
	KYLIN_USB_CDC_DATA,						///< 0x0a 通信类数据设备
	KYLIN_USB_CSCID,						///< 0x0b 智能卡
	KYLIN_USB_CONTENT_SEC,					///< 0x0d 内容安全
	KYLIN_USB_MISC,							///< 0xef 其他设备
	KYLIN_USB_APP_SPEC,						///< 0xfe 应用程序
	KYLIN_USB_VENDOR_SPEC,					///< 0xff 厂家自动以
	KYLIN_USB_PERSONAL_HEALTH,				///< 0x0f 个人医疗(linux内核没有该类型，为兼容windows)
	KYLIN_USB_DIAGNOSTIC,					///< 0xdc 诊断设备(linux内核没有该类型，为兼容windows)
	KYLIN_USB_MAX
};

/**  优先级的级别枚举 */
enum kylin_devctl_level {
	KYLIN_DEVCTL_LOW_LEVEL = 0,				///< 低等级
	KYLIN_DEVCTL_MIDDLE_LEVEL,				///< 中等级
	KYLIN_DEVCTL_HIGH_LEVEL,				///< 高等级
	KYLIN_LEVEL_MAX
};

/**  usb设备注册类型枚举 */
typedef enum kylin_devctl_usb_register {
	KYLIN_USB_NOT_REGISTER = 0,				///< 不注册(对应的设备默认放行)
	KYLIN_USB_REGISTER,						///< 注册
	KYLIN_USB_DEFAULT_UNCONTROL,			///< 默认不管控(即使usb接口禁用，该注册类型的设备也可用)
	KYLIN_REGISTER_MAX
}devctl_usb_register;

/**  设备管控策略的结构体 */
struct kylin_devctl_policy {
	int op;									///< 操作类型(见kylin_devctl_op_type枚举)
	int type;								///< 设备类型(见kylin_devctl_device_type枚举)
	int usb_type;							///< usb设备的类型(见kylin_devctl_usb_type枚举)
	unsigned int pid;						///< usb设备的pid
	unsigned int vid;						///< usb设备的vid
	int perm;								///< 设备权限(见kylin_device_perm枚举)
	int special_type;						///< usb设备的usb_type
	char serial[KYLIN_DEVCTL_CHAR_MID];		///< usb设备的序列号
	int version;							///< 版本号
};

/**  黑名单的结构体 */
struct kylin_blacklist_policy {
	int op;									///< 操作类型(见kylin_devctl_op_type枚举)
	blist_id_type type;						///< 黑名单类型(见kylin_blacklist_id_type枚举)
	char name[KYLIN_DEVCTL_CHAR_MID];		///< 黑名单名称
	int perm;								///< 黑名单的权限
};

/**  usb细粒度管控策略的结构体 */
struct kylin_usblist_policy {
	int op;									///< 操作类型(见kylin_devctl_op_type枚举)
	int usb_type;							///< usb设备的类型(见kylin_devctl_usb_type枚举)
	unsigned int usb_if_subclass;			///< usb设备的subclass类型
	unsigned int usb_if_protocol;			///< usb设备的protocol类型
	int perm;								///< 权限(见kylin_device_perm枚举)
};

/**  优先级的结构体 */
struct kylin_devctl_priority {
	int priority[KYLIN_PRIORITY_MAX];		///< 优先级数组
};

/**  usb注册的结构体 */
struct kylin_devctl_register {
	int usb_class;							///< usb设备的类型(见kylin_devctl_usb_type枚举)
	devctl_usb_register register_type;		///< 注册的类型(见kylin_devctl_usb_register枚举)
};

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 设备管控策略写入配置文件
 * @param {kylin_devctl_policy} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_set_device_policies_conf(struct kylin_devctl_policy *data, int count);

/**
 * @brief 黑名单策略写入配置文件
 * @param {kylin_blacklist_policy} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_set_blacklist_policies_conf(struct kylin_blacklist_policy *data, int count);

/**
 * @brief 优先级策略写入配置文件
 * @param {kylin_devctl_priority} *data
 * @return {int} 0： 成功， 非0： 失败
 */
int kylin_devctl_set_priority_policies_conf(struct kylin_devctl_priority *data);

/**
 * @brief usb设备注册信息写入配置文件
 * @param {kylin_devctl_register} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_set_register_policies_conf(struct kylin_devctl_register *data, int count);

/**
 * @brief usb设备细粒度管控策略写入配置文件
 * @param {kylin_usblist_policy} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_set_usblist_policies_conf(struct kylin_usblist_policy *data, int count);

/**
 * @brief 设置内核模块设备管控的状态
 * @param {int} status
 * @return {int} 0： 成功， 非0： 失败
 */
int kylin_devctl_set_status(int status);

/**
 * @brief 获取内核模块设备管控的状态
 * @return {int} 状态值
 */
int kylin_devctl_get_status(void);

/**
 * @brief 同步设备管控策略到内核模块
 * @param {kylin_devctl_policy} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_communicate_module_policy(struct kylin_devctl_policy *data, int count);

/**
 * @brief 同步黑名单策略到内核模块
 * @param {kylin_blacklist_policy} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_communicate_module_blacklist(struct kylin_blacklist_policy *data, int count);

/**
 * @brief 同步优先级策略到内核模块
 * @param {kylin_devctl_priority} *data
 * @return {int} 0： 成功， 非0： 失败
 */
int kylin_devctl_communicate_module_priority(struct kylin_devctl_priority *data);

/**
 * @brief 同步usb注册信息到内核模块
 * @param {kylin_devctl_register} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_communicate_module_register(struct kylin_devctl_register *data, int count);

/**
 * @brief 同步usb细粒度管控策略到内核模块
 * @param {kylin_usblist_policy} *data
 * @param {int} count	策略个数
 * @return {int} 操作失败的策略个数， 0： 成功
 */
int kylin_devctl_communicate_module_usblist(struct kylin_usblist_policy *data, int count);

/**
 * @brief 设备管控策略实时生效
 * @param {kylin_devctl_policy} *data
 * @param {int} count	策略个数
 * @return {int} 0： 成功， 非0： 失败
 */
int kylin_devctl_policies_real_time_effect(struct kylin_devctl_policy *data, int count);

/**
 * @brief 检查当前是否加载了kylin_devctl模块
 * @return {int} 1: 已加载，0：未加载
 */
int check_curent_kylin_devctl_module(void);

/**
 * @brief  获取所有黑名单策略
 * @param {int} *count
 * @return {struct} 结构体
 */
struct kylin_blacklist_policy *kylin_devctl_get_all_blacklist(int *count);

/**
 * @brief 获取所有usb注册信息
 * @param {int} *count
 * @return {struct} 结构体
 */
struct kylin_devctl_register *kylin_devctl_get_all_register(int *count);

/**
 * @brief	获取所有usb细粒度管控策略
 * @param {int} *count
 * @return {struct} 结构体
 */
struct kylin_usblist_policy *kylin_devctl_get_all_usblist(int *count);

/**
 * @brief 获取所有设备管控策略
 * @param {int} *count
 * @return {struct} 结构体
 */
struct kylin_devctl_policy *kylin_devctl_get_all_policies(int *count);

/**
 * @brief 获取指定设备的管控策略
 * @param {int} *count
 * @return {struct} 结构体
 */
struct kylin_devctl_policy *kylin_devctl_get_special_dev_policies(int *count);

#ifdef __cplusplus
}
#endif

#endif /* _KYLIN_DEVCTL_MODULE_H_ */
